# Purpose of the repo
This repo holds a script that takes data from the fbis interface design specificaiton and fbis DDS attachment and derives data equivalent to the dln parametrization files.

# Structure of the repo
```
.
├── data_src
│   ├── csv
│   ├── excel
│   └── xml
├── modules
│   ├── database
│   │   └── queries
│   └── datasources
│       ├── check
│       ├── get
│       └── parse
├── configuration.py
├── main.py
└── other.py
```
The above tree shows the strucure of the repository.
## data_src dir
```
.
├── csv
│   ├── fast_frame_tbl.csv
│   └── latching_rules.csv
├── excel
│   ├── ESS-0715539 - FBIS interface design specification.xlsx
│   └── ESS-3049041 - Attachment-01 FBIS-Signal Configuration.xlsx
└── xml
    └── chess.xml
```
The above tree shows the structre of the data_src directory. The data_src directory is used for storing datasources. An example of a datasource is the ESS-0715539 - FBIS interface design specification.xlsx excel workbook. The directory has three sub directories
1. csv
2. excel
3. xml

### csv dir
The csv directory is used for storing data in a csv file format. Currently there are two .csv files that should exist for the main script to run. The files are
1. fast_frame_tbl.csv
2. latching_rules.csv

The fast_frame_tbl.csv contains data on the number the number of "fast frame OK/NOK signals" and number of "NDS". This data was manually copied from the macros sheet of a dln parametrization excel workbook. The latching_rules.csv file contains data about the default latch mode of a signal. For example, Beam Permit signals have "latched NOK and ERR" as a default latch mode. The data was manually copied from the "General rules" sheet of the ESS-3049041 - Attachment-01 FBIS-Signal Configuration.xlsx excel workbook. A script can eventually be made to extract this data from ESS-3049041 - Attachment-01 FBIS-Signal Configuration.xlsx.

### excel
The excel subdirectory contains two workbooks
1. ESS-0715539 - FBIS interface design specification.xlsx
2. ESS-3049041 - Attachment-01 FBIS-Signal Configuration.xlsx
these workbooks need to be manually placed in the subdirectory for the main script to work properly.

### xml
The xml subdirectory holds xml data. It only contains chess.xml file. This file is automatically downloaded from a source specified in the [configuarion.py](configuration.py) file. 


## Modules directory
```
.
├── database
│   ├── __init__.py
│   ├── initialize.py
│   └── queries
│       ├── queries.py
│       └── user_defined_function.py
└── datasources
    ├── check
    ├── get
    │   ├── __init__.py
    │   ├── chess_xml.py
    │   ├── fbis_interface_design_spec.py
    │   ├── fbis_interfaces.py
    │   ├── fbis_signal_configuration.py
    │   └── get.py
    ├── initialize.py
    └── parse
        ├── fbis_interface_design_spec.py
        ├── fbis_interfaces.py
        ├── fbis_signal_configuration.py
        ├── parse.py
        └── utilities.py
```
The above tree shows the structure of the modules directory. The directory is broken into two subdirectories called database and datasources. 

### modules/datasources directory
The datasources directory is broken into a get subdirectory and and parse subdirectory. Notice that both subdirectories contain the same file names e.g. fbis_interface_design_spec.py, fbis_interfaces.py and fbis_singal_configuration.py. This is done because the procedure for getting a datasource and parsing a datasource vary from datasource to datasource. If data from a new datasource was needed then a new script for that datasource would need to be made in the get and parse sub directories.

#### modules/datasources/get
The get subdirectory conatins scripts that get a datasource to the correct place in the data_src directory. See [data_src dir](#data_src-dir) for more information. So the [chess_xml.py](modules/datasources/get/chess_xml.py) in the get subdirectory has a get method that downloads the data to the correct position. In this case of the [fbis_interface_design_spec.py](modules/datasources/get/fbis_interface_design_spec.py) and [fbis_signal_configuration.py](modules/datasources/get/fbis_signal_configuration.py), their get methods just checks whether their datasource exists; if their datasource does not exist an error is raised and information about how to get the datasource is provided.

##### modules/datasource/get/chess_xml.py
downloads the chess.xml file in the data_src/xml/ directory

##### modules/datasource/get/fbis_inteface_design_spec.py
specifies where to get ESS-0715539 - FBIS interface design specification.xlsx

##### modules/datasource/get/fbis_signal_configuration.py
specifices where to get ESS-3049041 - Attachment-01 FBIS-Signal Configuration.xlsx

##### modules/datasource/get/fbis_interfaces.py
gets fbis_interfaces data from the chess.xml file and stores it in the data_src/csv/fbis_interfaces.csv

##### modules/datasource/get/get.py
Provides a master get procudure that links all the other get methods.

#### parse directory
The parse directory contains scripts that parse a datasource and extract relevant information. All the parse methods return a pandas dataframe.

##### modules/datasource/parse/fbis_interface_Design_spec.py
Parses the FBIS Actuators Connections sheet and FBIS SS Connections sheet in ESS-0715539 - FBIS interface design specification.xlsx and returns a dataframe that is the union of the data from the two sheets.

##### modules/datasource/parse/fbis_interfaces.py
Parses the data_src/csv/fbis_interfaces.csv

##### modules/datasource/parse/fbis_signal_configuration.py
Parses ESS-3049041 - Attachment-01 FBIS-Signal Configuration.xlsx. It parses data from all the device sheets and returns a dataframe that is a union of all the sheets.

##### modules/datasource/parse/parse.py
Provides a master parse procedure that links all the other parse methods.

##### modules/datasource/parse/utilities.py
Contains procedures that are shared among the parse scripts.

#### modules/datasource/initialize.py
Provides a procedure that contructs initializes the data_src directory structure.

### modules/database
The database directory contains scripts that initializes a sqlite3 database with the parsed data from datasources in data_src; and queries to query the database.

#### modules/database/initialize.py
Provides method for initializing a sqlite3 database.

#### modules/database/queries
The queries directory provides the query that derives the dln parametrization file data from the tables in the sqlite3 database.

##### modules/database/queries/queries.py
Contains the query that derives the dln parametrization file data. 

##### modules/database/queries/user_defined_function.py
Contains many different helper functions that are used in the sql queries

## configuration.py
The [configuraiton.py](configuration.py) file contains the paths of all the expected datasources.

## main.py
The [main.py](main.py) script contains the main function that will 
1. check if all the datasources are there.
2. initialize an sqlite3 database.
3. query the database to derive the dln parametrization data

## other.py
The [other.py](other.py) script takes the data from data_src/csv/fast_frame_tbl.csv and data_src/csv/latching_rules exports the data to the sqlite3 database. The scripts need to be integrated into framework because it is awkward. Note data_src/csv/fast_frame_tbl.csv and data_src/csv/lathcing_rules do not exist in the configuration.py

# Running the script
First run
```
python3 main.py
```
This will give assertion errors because the excel datasources do not exist. Download the excel files specified in the error and place them in the expected path.
Run
```
python3 main.py
```
again. This time database.db will be created however errors will occur. This is expected. Run the 
```
python3 other.py
```
Then run
```
python3 main.py
```
The script should run without errors.


