# Latch rules queries
latch_rules_sql = '''
    SELECT encode_latch_rules(Signal,Type) as encoded_latch_rules, Description
    FROM latching_rules
'''
#---------------------------------------------------------------------------------------------------------------------------------------
# FBIS DDS queries
fbis_signal_base_sql = f'''
    SELECT signal.system_name, signal.signals_signal_name, signal.signals_signal_type, 
           signal.filtering_beam_destination, signal.filtering_beam_mode_,
           signal.masking_beam_destination, signal.masking_beam_mode_,
           signal.latching_latchable,
           make_pu_type(signal.signals_signal_name) as pu_type,
           encode_latch_rules(signal.signals_signal_name,signal.signals_signal_type) AS encoded_latch_rules
    FROM fbis_signal_configuration AS signal
    WHERE signal.signals_signal_type NOT LIKE 'Signal%' 
    AND signal.signals_signal_name NOT LIKE 'notes%' '''

fbis_signal_sql = f'''
    SELECT signal.system_name, signal.signals_signal_name, signal.signals_signal_type, 
           signal.filtering_beam_destination, signal.filtering_beam_mode_,
           signal.masking_beam_destination, signal.masking_beam_mode_,
           signal.latching_latchable,
           latch.Description AS latch_default,
           make_pu_type(signal.signals_signal_name) as pu_type
    FROM ({fbis_signal_base_sql}) AS signal
    INNER JOIN ({latch_rules_sql}) AS latch 
        ON signal.encoded_latch_rules = latch.encoded_latch_rules '''


#---------------------------------------------------------------------------------------------------------------------------------------
# FBIS interface design specification queries
## query for getting slink index 
dln_scu_sql = f'''
    SELECT DISTINCT dln_ess_name, scu_ess_name
    FROM fbis_interface_design_specification
    WHERE scu_ess_name NOT LIKE ''
    ORDER BY dln_ess_name
'''

dln_scu_sql_slink_index = f'''
    SELECT dln_ess_name, scu_ess_name, 
    ROW_NUMBER() OVER (PARTITION BY dln_ess_name ORDER BY scu_ess_name) - 1 AS slink_index
    FROM ({dln_scu_sql})
'''

## Base query
ess_name = "element_interlocked_by_fbis_without_duplicate_ea_inputs AS ess_name"
system_name = "system_name"
fbs = "element_fbs_the_lbs_is_the_same_as_the_sensor_system_rack_lbs AS fbs"
cable_name = "cable_ess_number"
dln_name = "dln_ess_name"
scu_name = "scu_ess_name"
scu_slot = "scu_slot"
scu_sub_slot = "scu_sub_slot"
card_type = "card_type"
interface_design_spec_fields = ", ".join([scu_name,scu_slot,dln_name,scu_sub_slot,card_type,cable_name,system_name,ess_name,fbs])

fbis_interface_design_base_sql = f'''
    SELECT {interface_design_spec_fields}
    FROM fbis_interface_design_specification
    WHERE ess_name NOT LIKE '%(datalink)%'
'''


## Getting data for base index
with_fast_frame = f'''
    SELECT connection.ess_name, connection.fbs, connection.cable_ess_number,
           connection.dln_ess_name,
           connection.scu_ess_name, connection.scu_slot, connection.scu_sub_slot,
           connection.card_type, 
           ff.fast_frames,ff.num_nds
    FROM ({fbis_interface_design_base_sql}) AS connection 
    INNER JOIN fast_frame AS ff ON connection.card_type = ff.mc_type
'''

data_for_base_index = f'''
    SELECT DISTINCT connection.scu_ess_name, connection.scu_slot,
           connection.fast_frames, connection.num_nds
    FROM ({with_fast_frame}) AS connection
'''

base_index = f'''
    SELECT scu_ess_name,
           scu_slot,
           SUM(fast_frames) OVER (PARTITION BY scu_ess_name ORDER BY scu_slot) - fast_frames AS base_index_ff,
           SUM(num_nds) OVER (PARTITION BY scu_ess_name ORDER BY scu_slot) - num_nds AS base_index_nd
    FROM ({data_for_base_index}) 
'''


## Main query
fbis_interface_design_sql = f'''
    SELECT connection.ess_name,connection.system_name,connection.fbs,
           connection.cable_ess_number, connection.dln_ess_name,
           connection.scu_ess_name,connection.scu_slot,
           connection.scu_sub_slot, connection.card_type,
           dln_scu.slink_index,
           base_index.base_index_ff,base_index.base_index_nd
    FROM ({fbis_interface_design_base_sql}) AS connection
    INNER JOIN ({dln_scu_sql_slink_index}) AS dln_scu ON connection.scu_ess_name = dln_scu.scu_ess_name
    INNER JOIN ({base_index}) AS base_index ON connection.scu_ess_name || connection.scu_slot = base_index.scu_ess_name || base_index.scu_slot
'''
#---------------------------------------------------------------------------------------------------------------------------------------



signal_connection = f'''
    SELECT signal.system_name,
           signal.signals_signal_name, signal.signals_signal_type,
           signal.filtering_beam_destination, signal.filtering_beam_mode_, 
           signal.latching_latchable, signal.latching_latchable,
           signal.masking_beam_destination, signal.masking_beam_mode_,
           signal.pu_type,
           signal.latch_default,
           connection.ess_name,
           connection.dln_ess_name,
           connection.slink_index,
           connection.scu_ess_name,
           connection.scu_sub_slot,
           connection.base_index_ff,
           connection.base_index_nd,
           ROW_NUMBER() OVER (ORDER BY connection.dln_ess_name, connection.scu_ess_name, connection.scu_slot, connection.scu_sub_slot) AS row_num
    FROM ({fbis_interface_design_sql}) AS connection 
    INNER JOIN ({fbis_signal_sql}) as signal ON signal.system_name = connection.system_name
    ORDER BY connection.dln_ess_name, connection.scu_ess_name, connection.scu_slot, connection.scu_sub_slot
'''


dln_test_data_sql = f'''
    SELECT interface_signal_connection.system_name,
           make_alias_field(interface_signal_connection.system_name,interface_signal_connection.signals_signal_name,interface_signal_connection.ess_name) as alias,
           interface_signal_connection.pu_type || ROW_NUMBER() OVER (PARTITION BY interface_signal_connection.dln_ess_name, interface_signal_connection.pu_type) AS pu_index,
           interface_signal_connection.slink_index AS slink_index,
           make_slink_origin_fast_field(interface_signal_connection.signals_signal_type) as slink_origin_fast,
           make_base_index(interface_signal_connection.system_name,
                           interface_signal_connection.signals_signal_name,
                           interface_signal_connection.signals_signal_type,
                           interface_signal_connection.scu_sub_slot,
                           interface_signal_connection.base_index_ff,
                           interface_signal_connection.base_index_nd) AS signal_index,
           make_latch_enable_field(interface_signal_connection.latching_latchable) as latch_enable,
           make_latch_default_field(interface_signal_connection.latching_latchable,interface_signal_connection.latch_default) AS latch_default,
           make_filter_conditions_pbd_field(interface_signal_connection.filtering_beam_destination) AS filter_conditions_pbd,
           make_filter_conditions_pbm_field(interface_signal_connection.filtering_beam_mode_) AS filter_conditions_pbm,
           make_filter_enable_field(interface_signal_connection.filtering_beam_destination,interface_signal_connection.filtering_beam_mode_) as filter_enable,
           make_mask_conditions_pbd_field(interface_signal_connection.masking_beam_destination) AS mask_conditions_pbd,
           make_mask_conditions_pbm_field(interface_signal_connection.masking_beam_mode_) AS mask_conditions_pbm,
           make_mask_enable_field() as mask_enable,
           interface_signal_connection.dln_ess_name AS dln_ess_name
    FROM ({signal_connection}) AS interface_signal_connection
    WHERE interface_signal_connection.dln_ess_name LIKE '%DLN04%'
    ORDER BY interface_signal_connection.row_num
    '''