from pathlib import Path
import xml.etree.ElementTree as ET

def give_index_to_data(ess_name:str,counters:list)->tuple:
    if "RFS-FIM" in ess_name:
        identifier = "RFLPS " + f"{counters[0]:02d}"
        counters[0] += 1
        return (identifier,counters)
    elif "ICBLM" in ess_name:
        identifier = "ICBLM " + f"{counters[1]:02d}"
        counters[1] += 1
        return (identifier,counters)
    elif "nBLM" in ess_name:
        identifier = "nBLM " + f"{counters[2]:02d}"
        counters[2] += 1
        return (identifier,counters)
    elif "BPM" in ess_name:
        identifier = "BPM " + f"{counters[3]:02d}"
        counters[3] += 1
        return (identifier,counters)
    elif "BCM" in ess_name:
        identifier = bcm_names[counters[4]-1]
        counters[4] += 1
        return (identifier,counters)
    else:
        return ("",counters)

bcm_names = ["BCM 01","BCM 01R","BCM 02", "BCM 02R",
             "BCM 03","BCM 04","BCM 05","BCM 06",
             "BCM 07","BCM 08"]


def is_rf_data(element):
    ess_name = element.attrib.get("essName")
    iso_class = element.attrib.get("isoClass")
    if ess_name and ("RFS-FIM" in ess_name):
        return ("Input - Output" in iso_class)
    else:
        return False

def is_bpm_data(element):
    ess_name = element.attrib.get("essName")
    if ess_name and ("BPM" in ess_name):
        return "AMC" in ess_name
    else:
        return False

def is_blm_data(element):
    ess_name = element.attrib.get("essName")
    if ess_name and ("BLM" in ess_name):
        return "AMC" in ess_name
    else:
        return False

def is_bcm_data(element):
    ess_name = element.attrib.get("essName")
    if ess_name and ("PBI-BCM0" in ess_name):
        return ("AMC" in ess_name)
    else:
        return False

def is_append_data(element)->bool:
    checks = [is_rf_data,is_bpm_data,
              is_blm_data,is_bcm_data]
    for check in checks:
        if check(element):
            return True
    return False


def extract_fbis_interfaces_chess_xml(chess_xml_path:Path,
                                      fbis_interfaces:Path)->None:
    tree = ET.parse(str(chess_xml_path))
    root = tree.getroot()
    new_root = ET.Element('root')
    for elem in root.iter():
        if is_append_data(elem):
            new_root.append(elem)
    counters = [1,1,1,1,1]

    file_handle = open("data_src/xml/test.xml","wb")
    tree = ET.ElementTree(new_root)
    tree.write(file_handle)
    file_handle.close()

    with open(fbis_interfaces,'w') as file:
        file.write("ess_name,tag,system_name\n")
        for elem in new_root.iter():
            ess_name = elem.attrib.get("essName")
            tag = elem.attrib.get("tag")
            if ess_name:
                (identifier,counters) = give_index_to_data(ess_name,counters)
                file.write(f"{ess_name},{tag},{identifier}\n")
    return None

def get(data_src:Path)->None:
    from modules.datasources.get.chess_xml import get
    from configuration import chess_xml
    get(chess_xml)
    extract_fbis_interfaces_chess_xml(chess_xml,data_src)
    return None