from pathlib import Path
import pandas as pd

from modules.datasources.parse.utilities import make_csv_name
from modules.datasources.parse.utilities import rename_fields




def parse_general_rules(data_path:Path)->pd.DataFrame:
    data_path_string = str(data_path)
    split_by_slash = data_path_string.split("/")
    workbook_path = "/".join(split_by_slash[:-1])
    sheetname = split_by_slash[-1]
    excel_file = pd.ExcelFile(workbook_path)
    df = excel_file.parse(sheetname,nrows=11,skiprows=7)
    return df


def parse_device_sheets(data_path:Path)->pd.DataFrame:
    excel_file = pd.ExcelFile(data_path)
    dfs = []
    skip_sheets = ["General Rules","Cover Sheet","Others","Options"]
    special_sheet = ["BCMs"]
    for sheet in excel_file.sheet_names:
        if sheet in skip_sheets:
            pass
        elif sheet in special_sheet:
            df = excel_file.parse(sheet,nrows=78)
            df["Device_type"] = sheet
            df = rename_fields(df).reindex()
            dfs.append(df)
        else:
            df = excel_file.parse(sheet)
            df["Device_type"] = sheet
            df = rename_fields(df).reindex()
            dfs.append(df)
    
    union_sheets = pd.concat(dfs)
    union_sheets = union_sheets.ffill()
    # union_sheets.to_csv(make_csv_name(data_path),index=False)
    return union_sheets


# def parse(data_path:Path)->pd.DataFrame:
#     excel_file = pd.ExcelFile(data_path)
#     dfs = []
#     skip_sheets = ["General Rules","Cover Sheet","Others","Options"]
#     special_sheet = ["BCMs"]
#     for sheet in excel_file.sheet_names:
#         if sheet in skip_sheets:
#             pass
#         elif sheet in special_sheet:
#             df = excel_file.parse(sheet,nrows=78)
#             df["Device_type"] = sheet
#             df = rename_fields(df).reindex()
#             dfs.append(df)
#         else:
#             df = excel_file.parse(sheet)
#             df["Device_type"] = sheet
#             df = rename_fields(df).reindex()
#             dfs.append(df)
    
#     union_sheets = pd.concat(dfs)
#     union_sheets = union_sheets.ffill()
#     # union_sheets.to_csv(make_csv_name(data_path),index=False)
#     return union_sheets

def parse(data_path:Path)->pd.DataFrame:
    data_path_string = str(data_path)
    if "General Rule" in data_path_string:
        return parse_general_rules(data_path)
    else:
        return parse_device_sheets(data_path)

