from typing import List
import pandas as pd
from pathlib import Path

def name_unnamed_fields(field_names:List[str])->List[str]:
    for i in range(len(field_names)-1):
        ptr1 = field_names[i]
        ptr2 = field_names[i+1]
        if "Unnamed" in ptr2:
            field_names[i+1] = ptr1
    return field_names

def combine_field_names(field_names:List[str],extra_data:List[str]):
    columns = []
    for a,b in zip(field_names,extra_data):
        if "float" in str(type(b)):
            columns.append(a)
        elif a == "Device_type":
            columns.append(a)
        else:
            column = a + "_" + b
            columns.append(column)
    return columns

def is_repeated_names(field_names:List[str]) -> bool:
    return len(set(field_names)) != len(field_names)

def get_field_names(df:pd.DataFrame)->List[List[str]]:
    field_names = name_unnamed_fields(df.columns.values.tolist())
    extra_data = df.iloc[0].tolist()
    field_names = combine_field_names(field_names,extra_data)
    field_names = process_field_names(field_names)
    assert not is_repeated_names(field_names), f"There are repeated field names in {field_names}"
    return field_names
    
def rename_fields(df:pd.DataFrame)->pd.DataFrame:
    field_names = get_field_names(df)
    df.columns = field_names
    return df

def make_csv_name(path:Path)->str:
    assert "xlsx" in str(path), f"path must be to an excel file, not {path}"
    split_slash = str(path).split("/")
    split_dot = split_slash[-1].split(".")
    return f"data_src/csv/{split_dot[0]}.csv"

def process_field_name(field_name:str)->str:
    new_name = field_name.replace(" ","_").replace("\n","_").lower()
    new_name = new_name.replace("(","").replace(")","")
    new_name = new_name.replace("*","")
    new_name = new_name.replace("[","").replace("]","")
    return new_name

def process_field_names(field_names:List[str])->List[str]:
    return list(map(process_field_name,field_names))

def give_index_to_data(ess_name:str,counters:list)->tuple:
    if "RFS-FIM" in ess_name:
        identifier = "RFLPS " + f"{counters[0]:02d}"
        counters[0] += 1
        return (identifier,counters)
    elif "ICBLM" in ess_name:
        identifier = "ICBLM " + f"{counters[1]:02d}"
        counters[1] += 1
        return (identifier,counters)
    elif "nBLM" in ess_name:
        identifier = "nBLM " + f"{counters[2]:02d}"
        counters[2] += 1
        return (identifier,counters)
    elif "BPM" in ess_name:
        identifier = "BPM " + f"{counters[3]:02d}"
        counters[3] += 1
        return (identifier,counters)
    elif "BCM" in ess_name:
        identifier = bcm_names[counters[4]-1]
        counters[4] += 1
        return (identifier,counters)
    else:
        return ("",counters)

bcm_names = ["BCM 01","BCM 01R","BCM 02", "BCM 02R",
             "BCM 03","BCM 04","BCM 05","BCM 06",
             "BCM 07"]